@@ -1,6 +1,6 @@
 # -*- python -*-
 #
-# Copyright (C) 2001,2002,2003 Jason R. Mastaler <jason@mastaler.com>
+# Copyright (C) 2001-2007 Jason R. Mastaler <jason@mastaler.com>
 #
 # This file is part of TMDA.
 #
@@ -23,31 +23,28 @@
 
 
 from cStringIO import StringIO
-from string import whitespace as WHITESPACE
 import cPickle
 import email
-import email.Utils
+import email.utils
 import fileinput
 import fnmatch
 import os
 import popen2
-import pwd
 import re
 import socket
 import stat
 import sys
 import tempfile
+import textwrap
 import time
 
 import Errors
 
 
-DOT = '.'
-EMPTYSTRING = ''
 MODE_EXEC = 01
 MODE_READ = 04
 MODE_WRITE = 02
-NL = '\n'
+POSIX_NAME_MAX = 255                    # maximum length of a file name
 
 
 def gethostname():
@@ -67,6 +64,7 @@ def getfullname():
                os.environ.get('NAME') or \
                os.environ.get('MAILNAME')
     if not fullname:
+        import pwd
         fullname = pwd.getpwuid(os.getuid())[4]
         if fullname:
             fullname = fullname.split(',')[0]
@@ -80,6 +78,7 @@ def getusername():
                os.environ.get('USER') or \
                os.environ.get('LOGNAME')
     if not username:
+        import pwd
         username = pwd.getpwuid(os.getuid())[0]
     if not username:
         username = '<unknown>'
@@ -88,16 +87,19 @@ def getusername():
 
 def getuid(username):
     """Return username's numerical user ID."""
+    import pwd
     return pwd.getpwnam(username)[2]
 
 
 def getgid(username):
     """Return username's numerical group ID."""
+    import pwd
     return pwd.getpwnam(username)[3]
 
 
 def gethomedir(username):
     """Return the home directory of username."""
+    import pwd
     return pwd.getpwnam(username)[5]
 
 
@@ -127,6 +129,13 @@ def getfileuid(path):
     return statinfo[stat.ST_UID]
 
 
+def issticky(path):
+    """Return True if the sticky bit is set on path.  Generally only
+    appropriate for directories."""
+    statinfo = os.stat(path)
+    return statinfo[stat.ST_MODE] & stat.S_ISVTX
+
+
 def getvdomainprepend(address, vdomainsfile):
     ret_prepend = ''
     if os.path.exists(vdomainsfile):
@@ -175,7 +184,7 @@ def getvuserhomedir(user, domain, script):
 
 def getuserparams(login):
     "Return a user's home directory, UID, & GID."
-
+    import pwd
     stats = pwd.getpwnam(login)
     return (stats[5], stats[2], stats[3])
 
@@ -235,20 +244,8 @@ def format_timeout(timeout):
     if not match:
         return timeout
     (num, unit) = match.groups()
-    if unit == 'Y':
-        timeout = num + " years"
-    elif unit == 'M':
-        timeout = num + " months"
-    elif unit == 'w':
-        timeout = num + " weeks"
-    elif unit == 'd':
-        timeout = num + " days"
-    elif unit == 'h':
-        timeout = num + " hours"
-    elif unit == 'm':
-        timeout = num + " minutes"
-    else:
-        timeout = num + " seconds"
+    import Defaults
+    timeout = num + " " + Defaults.TIMEOUT_UNITS[unit]
     if int(num) == 1:
         timeout = timeout[:-1]
     return timeout
@@ -294,17 +291,16 @@ def make_msgid(timesecs=None, pid=None):
 
 
 def make_date(timesecs=None):
-    """Return an RFC 2822 compliant Date: string.  e.g,
+    """Return an RFC 2822 compliant Date string relative to the local
+    timezone.  e.g,
 
-    Thu, 24 Apr 2003 10:15:15 +1200 (NZST)
+    Tue, 02 Mar 2004 15:55:05 +1300
 
     timesecs is optional, and if not given, the current time is used.
     """
     if timesecs is None:
         timesecs = time.time()
-    tzname = time.tzname[time.localtime(timesecs)[-1]]
-    basedate = email.Utils.formatdate(timesecs, localtime=1)
-    return '%s (%s)' % (basedate, tzname)
+    return email.utils.formatdate(timesecs, localtime=True)
 
 
 def file_to_dict(file, dict):
@@ -424,21 +420,20 @@ def append_to_file(str, fullpathname):
     file.close()
 
 
-def pager(file):
-    """Display file using a UNIX text pager such as less or more."""
-    pager_list = []
+def pager(str):
+    """Display a string using a UNIX text pager such as less or more."""
     pager = os.environ.get('PAGER')
     if pager is None:
         # try to locate less or more if $PAGER is not set
         for prog in ('less', 'more'):
             path = os.popen('which ' + prog).read()
-            if path != '':
+            if path <> '':
                 pager = path
                 break
-    for arg in pager.split():
-        pager_list.append(arg)
-    pager_list.append(file)
-    os.spawnvp(os.P_WAIT, pager_list[0], pager_list)
+    try:
+	os.popen(pager, 'w').write(str)
+    except IOError:
+	return
 
 
 def normalize_sender(sender):
@@ -449,10 +444,14 @@ def normalize_sender(sender):
       outside the directory.
     - Spaces are replaced with underscores.
     - The address is lowercased.
+    - Truncate sender at 233 chars to insure the full filename
+    (including time, pid, and two dots) fits within the POSIX limit of
+    255 chars for a filename.
     """
     sender = sender.replace(' ', '_')
     sender = sender.replace('/', ':')
-    return sender.lower()
+    sender = sender.lower()
+    return sender[:POSIX_NAME_MAX - 22]
 
 
 def confirm_append_address(xp, rp):
@@ -461,10 +460,12 @@ def confirm_append_address(xp, rp):
     rp is the envelope sender address.
 
     Compare the two addresses, and return the address appropriate for
-    CONFIRM_APPEND use based on the PRIMARY_ADDRESS_MATCH setting.
+    CONFIRM_APPEND based on the PRIMARY_ADDRESS_MATCH setting.
     """
     if not xp:
         return rp
+    if '@' not in rp or '@' not in xp:
+        return rp
     import Defaults
     rpl = rp.lower()
     xpl = xp.lower()
@@ -504,7 +505,7 @@ def confirm_append_address(xp, rp):
     return rp
 
 
-def msg_from_file(fp, strict=0):
+def msg_from_file(fp, fullParse=False):
     """Read a file and parse its contents into a Message object model.
     Replacement for email.message_from_file().
     
@@ -512,20 +513,24 @@ def msg_from_file(fp, strict=0):
     to parse the message body, instead setting the payload to the raw
     body as a string.  This is faster, and also helps us avoid
     problems trying to parse spam with broken MIME bodies."""
-    from email.Message import Message
-    from email.Parser import HeaderParser
-    msg = HeaderParser(Message, strict=strict).parse(fp)
-    msg.header_parsed = 1
+    from email.message import Message
+    if fullParse:
+	from email.parser import Parser
+	msg = Parser(Message).parse(fp)
+    else:
+	from email.parser import HeaderParser
+	msg = HeaderParser(Message).parse(fp)
+    #msg.header_parsed = True
     return msg
 
 
-def msg_as_string(msg, maxheaderlen=0, mangle_from_=0, unixfrom=0):
+def msg_as_string(msg, maxheaderlen=False, mangle_from_=False, unixfrom=False):
     """A more flexible replacement for Message.as_string().  The default
     is a textual representation of the message where the headers are
     not wrapped, From is not escaped, and a leading From_ line is not
     added.
 
-    msg is an email.Message.Message object.
+    msg is an email.message.Message object.
 
     maxheaderlen specifies the longest length for a non-continued
     header.  Disabled by default.  RFC 2822 recommends 78.
@@ -536,12 +541,13 @@ def msg_as_string(msg, maxheaderlen=0, mangle_from_=0, unixfrom=0):
 
     unixfrom forces the printing of the envelope header delimiter.
     Default is False."""
-    from email import Generator
+    from email import generator
     fp = StringIO()
-    if hasattr(msg, 'header_parsed') and msg.header_parsed:
-        genclass = Generator.HeaderParsedGenerator
-    else:
-        genclass = Generator.Generator
+    #if hasattr(msg, 'header_parsed') and msg.header_parsed:
+    #    genclass = Generator.HeaderParsedGenerator
+    #else:
+         #genclass = Generator.Generator
+    genclass = generator.Generator
     g = genclass(fp, mangle_from_=mangle_from_, maxheaderlen=maxheaderlen)
     g.flatten(msg, unixfrom=unixfrom)
     return fp.getvalue()
@@ -558,6 +564,26 @@ def sendmail(msgstr, envrecip, envsender):
     envsender is the envelope sender address.
     """
     import Defaults
+    # Sending mail with a null envelope sender address <> is not done
+    # the same way across the different supported MTAs, nor across the
+    # two mail transports (SMTP and /usr/sbin/sendmail).
+    #
+    # The most common method is to use the string '<>'.  There are two
+    # exceptions where an empty string must be used instead.
+    #
+    # 1) When running qmail/courier and using the sendmail transport.
+    # qmail munges the envelope sender address into <"<>"@domain.dom
+    # if "sendmail -f <>" is used.
+    #
+    # 2) When running Postfix and using the sendmail transport.
+    # Old versions of Postfix apparently also have trouble with
+    # "sendmail -f <>", though Postfix 2.0.x does not.
+    if envsender == '':
+        envsender = '<>'
+    if envsender == '<>' and \
+           Defaults.MAIL_TRANSFER_AGENT in ('postfix', 'qmail') and \
+           Defaults.MAIL_TRANSPORT == 'sendmail':
+        envsender = ''
     if Defaults.MAIL_TRANSPORT == 'sendmail':
         # You can avoid the shell by passing a tuple of arguments as
         # the command instead of a string.  This will cause the
@@ -580,20 +606,20 @@ def decode_header(str):
     """Accept a possibly encoded message header as a string, and
     return a decoded string if it can be decoded.
 
-    JRM: email.Header has a decode_header method, but it returns a
+    JRM: email.header has a decode_header method, but it returns a
     list of decoded pairs, one for each part of the header, which is
     an awkward interface IMO, especially when the header contains a
     mix of encoded and non-encoded parts.
     """
     try:
-        from email import Header
+        from email import header
         parts = []
-        pairs = Header.decode_header(str)
+        pairs = header.decode_header(str)
         for pair in pairs:
             parts.append(pair[0])
         decoded_string = ' '.join(parts)
         return decoded_string
-    except email.Errors.HeaderParseError:
+    except email.errors.HeaderParseError:
         return str
 
 
@@ -617,7 +643,7 @@ def headers_as_string(msg):
     try:
         hdrstr = '\n'.join(['%s: %s' %
                             (k, decode_header(v)) for k, v in msg.items()])
-    except email.Errors.HeaderParseError:
+    except email.errors.HeaderParseError:
         hdrstr = headers_as_raw_string(msg)
     return hdrstr
 
@@ -633,7 +659,7 @@ def body_as_raw_string(msg):
 def rename_headers(msg, old, new):
     """Rename all occurances of a message header in a Message object.
 
-    msg is an email.Message.Message object.
+    msg is an email.message.Message object.
 
     old is name of the header to rename.
 
@@ -649,7 +675,7 @@ def rename_headers(msg, old, new):
 def add_headers(msg, headers):
     """Add headers to a Message object.
 
-       msg is an email.Message.Message object.
+       msg is an email.message.Message object.
 
        headers is a dictionary of headers and values.
        """
@@ -664,7 +690,7 @@ def add_headers(msg, headers):
 def purge_headers(msg, headers):
     """Purge headers from a Message object.
 
-       msg is an email.Message.Message object.
+       msg is an email.message.Message object.
 
        headers is a list of headers.
        """
@@ -726,13 +752,28 @@ def build_dbm(filename):
         return 1
 
 
-def pickleit(object, file, bin=0):
+def pickleit(object, file, proto=2):
     """Store object in a pickle file.
-    Optional bin specifies whether to use binary or text pickle format."""
+
+    Optional 'proto' specifies which data storage format to use.
+    Possible integer values include:
+
+    0 (original ASCII protocol and is backwards compatible with
+    earlier versions of Python)
+
+    1 (old binary format which is also compatible with earlier
+    versions of Python)
+
+    2 (a more effecient binary format introduced in Python 2.3)
+
+    -1 (always choose the highest protocol version available)
+
+    default is 2, since we must support Python 2.3 and above.
+    """
     tempfile.tempdir = os.path.dirname(file)
-    tmpname = tempfile.mktemp()
+    tmpname = tempfile.mkstemp()[1]
     fp = open(tmpname, 'w')
-    cPickle.dump(object, fp, bin)
+    cPickle.dump(object, fp, proto)
     fp.close()
     os.rename(tmpname, file)
     return
@@ -792,83 +833,10 @@ def findmatch(list, addrs):
                         return 1
 
 
-def wraptext(text, column=70, honor_leading_ws=1):
-    """Wrap and fill the text to the specified column.
-
-    Wrapping is always in effect, although if it is not possible to
-    wrap a line (because some word is longer than `column' characters)
-    the line is broken at the next available whitespace boundary.
-    Paragraphs are also always filled, unless honor_leading_ws is true
-    and the line begins with whitespace.
-
-    Includes code from Mailman
-    <URL:http://www.gnu.org/software/mailman/mailman.html>
-    Copyright (C) 1998,1999,2000,2001 by the Free Software Foundation, Inc.,
-    and licensed under the GNU General Public License version 2.
-    """
-    wrapped = ''
-    # first split the text into paragraphs, defined as a blank line
-    paras = re.split('\n\n', text)
-    for para in paras:
-        # fill
-        lines = []
-        fillprev = 0
-        for line in para.split(NL):
-            if not line:
-                lines.append(line)
-                continue
-            if honor_leading_ws and line[0] in WHITESPACE:
-                fillthis = 0
-            else:
-                fillthis = 1
-            if fillprev and fillthis:
-                # if the previous line should be filled, then just append a
-                # single space, and the rest of the current line
-                lines[-1] = lines[-1].rstrip() + ' ' + line
-            else:
-                # no fill, i.e. retain newline
-                lines.append(line)
-            fillprev = fillthis
-        # wrap each line
-        for text in lines:
-            while text:
-                if len(text) <= column:
-                    line = text
-                    text = ''
-                else:
-                    bol = column
-                    # find the last whitespace character
-                    while bol > 0 and text[bol] not in WHITESPACE:
-                        bol = bol - 1
-                    # now find the last non-whitespace character
-                    eol = bol
-                    while eol > 0 and text[eol] in WHITESPACE:
-                        eol = eol - 1
-                    # watch out for text that's longer than the column width
-                    if eol == 0:
-                        # break on whitespace after column
-                        eol = column
-                        while eol < len(text) and \
-                              text[eol] not in WHITESPACE:
-                            eol = eol + 1
-                        bol = eol
-                        while bol < len(text) and \
-                              text[bol] in WHITESPACE:
-                            bol = bol + 1
-                        bol = bol - 1
-                    line = text[:eol+1] + '\n'
-                    # find the next non-whitespace character
-                    bol = bol + 1
-                    while bol < len(text) and text[bol] in WHITESPACE:
-                        bol = bol + 1
-                    text = text[bol:]
-                wrapped = wrapped + line
-            wrapped = wrapped + '\n'
-            # end while text
-        wrapped = wrapped + '\n'
-        # end for text in lines
-    # the last two newlines are bogus
-    return wrapped[:-2]
+def wraptext(text, column=70):
+    """Wrap and fill the text to the specified column width."""
+    wrapper = textwrap.TextWrapper(width=column, break_long_words=False)
+    return wrapper.fill(text)
 
 
 def maketext(templatefile, vardict):
